@@ -1,5 +1,76 @@
 '''
-# replace this
+# CDK Library for AWS SSO
+
+[![build](https://github.com/RenovoSolutions/cdk-library-aws-sso/actions/workflows/build.yml/badge.svg)](https://github.com/RenovoSolutions/cdk-library-aws-sso/actions/workflows/build.yml)
+
+**Note: This construct library is currently being developed and requires more tests, but fundamentally it should work.**
+
+This CDK library provides L2 constructs for the AWS SSO CfnPermissionSet and CfnAssignment. Assignment is not consumed by other resources so it's attributes and properties are minimal, but a L2 construct makes it easier to interact with and allows providing a more fully featured construct for the Permission Set and assignment requires.
+
+## Features
+
+* L2 Construct for `PermissionSet` including importing from ARN and granting the permission
+* L2 Construct for `Assignment`
+* Some enums to provide valid inputs for certain properties
+
+## API Doc
+
+See [API](API.md)
+
+## Examples
+
+### PermissionSet
+
+```python
+import { PermissionSet, Assignment, PrincipalTypes } from '@renovosolutions/cdk-library-aws-sso';
+import {
+  App,
+  Stack,
+  StackProps,
+  aws_sso as sso,
+  aws_iam as iam,
+  Duration,
+} from 'aws-cdk-lib';
+
+// create a permission set
+const permissionSetExample = new PermissionSet(this, 'permissionSet', {
+  ssoInstanceArn: 'arn:aws:sso:::instance/ssoins-1234567891234567',
+  name: 'ExamplePermissionSet',
+  description: 'Example permission set with some policies',
+  awsManagedPolicies:  [
+    iam.ManagedPolicy.fromAwsManagedPolicyName('job-function/ViewOnlyAccess'),
+  ],
+  customerManagedPolicyReferences: [
+    {
+      name: 'someServiceLogRead', // must exist in the target account
+      path: '/',
+    }
+  ],
+  relayStateType: 'https://us-east-1.console.aws.amazon.com/cloudwatch/home'
+})
+
+// assign it to an account/principal with an Assignment
+new Assignment(this, 'ExampleAssignment', {
+  permissionSet: permissionSetExample,
+  principal: {
+    principalId: '25750630-0ae9-479a-97c2-0afc2d5b4eac,
+    principalType: PrincipalTypes.GROUP,
+  },
+  targetId: '124567890123456',
+});
+
+// assign it to something else with a grant
+permissionSetExample.grant(
+  {
+    principalId: '12350630-0ae9-479a-97c2-0afc2d5b4eac',
+    principalType: PrincipalTypes.GROUP,
+  },
+  '344567890123456',
+);
+
+// import an existing permission set
+const existingPermissionSetExample = PermissionSet.fromPermissionSetArn(this, 'existingPermissionSetExample', 'arn:aws:sso:::permissionSet/ssoins-1234567891234567/ps-55a5555a5a55ab55');
+```
 '''
 import abc
 import builtins
@@ -50,6 +121,7 @@ class AssignmentAttributes:
         "permission_set": "permissionSet",
         "principal": "principal",
         "target_id": "targetId",
+        "target_type": "targetType",
     },
 )
 class AssignmentProps:
@@ -59,12 +131,14 @@ class AssignmentProps:
         permission_set: "IPermissionSet",
         principal: typing.Union["PrincipalProperty", typing.Dict[str, typing.Any]],
         target_id: builtins.str,
+        target_type: typing.Optional["TargetTypes"] = None,
     ) -> None:
         '''The properties of a new assignment.
 
         :param permission_set: The permission set to assign to the principal.
         :param principal: The principal to assign the permission set to.
         :param target_id: The target id the permission set will be assigned to.
+        :param target_type: The entity type for which the assignment will be created. Default: TargetTypes.AWS_ACCOUNT
         '''
         if isinstance(principal, dict):
             principal = PrincipalProperty(**principal)
@@ -73,11 +147,14 @@ class AssignmentProps:
             check_type(argname="argument permission_set", value=permission_set, expected_type=type_hints["permission_set"])
             check_type(argname="argument principal", value=principal, expected_type=type_hints["principal"])
             check_type(argname="argument target_id", value=target_id, expected_type=type_hints["target_id"])
+            check_type(argname="argument target_type", value=target_type, expected_type=type_hints["target_type"])
         self._values: typing.Dict[str, typing.Any] = {
             "permission_set": permission_set,
             "principal": principal,
             "target_id": target_id,
         }
+        if target_type is not None:
+            self._values["target_type"] = target_type
 
     @builtins.property
     def permission_set(self) -> "IPermissionSet":
@@ -100,6 +177,15 @@ class AssignmentProps:
         assert result is not None, "Required property 'target_id' is missing"
         return typing.cast(builtins.str, result)
 
+    @builtins.property
+    def target_type(self) -> typing.Optional["TargetTypes"]:
+        '''The entity type for which the assignment will be created.
+
+        :default: TargetTypes.AWS_ACCOUNT
+        '''
+        result = self._values.get("target_type")
+        return typing.cast(typing.Optional["TargetTypes"], result)
+
     def __eq__(self, rhs: typing.Any) -> builtins.bool:
         return isinstance(rhs, self.__class__) and rhs._values == self._values
 
@@ -225,11 +311,13 @@ class IPermissionSet(aws_cdk.IResource, typing_extensions.Protocol):
         self,
         principal: typing.Union["PrincipalProperty", typing.Dict[str, typing.Any]],
         target_id: builtins.str,
+        target_type: typing.Optional["TargetTypes"] = None,
     ) -> "Assignment":
         '''Grant this permission set to a given principal for a given targetId (AWS account identifier) on a given SSO instance.
 
         :param principal: -
         :param target_id: -
+        :param target_type: -
         '''
         ...
 
@@ -264,17 +352,20 @@ class _IPermissionSetProxy(
         self,
         principal: typing.Union["PrincipalProperty", typing.Dict[str, typing.Any]],
         target_id: builtins.str,
+        target_type: typing.Optional["TargetTypes"] = None,
     ) -> "Assignment":
         '''Grant this permission set to a given principal for a given targetId (AWS account identifier) on a given SSO instance.
 
         :param principal: -
         :param target_id: -
+        :param target_type: -
         '''
         if __debug__:
             type_hints = typing.get_type_hints(IPermissionSet.grant)
             check_type(argname="argument principal", value=principal, expected_type=type_hints["principal"])
             check_type(argname="argument target_id", value=target_id, expected_type=type_hints["target_id"])
-        return typing.cast("Assignment", jsii.invoke(self, "grant", [principal, target_id]))
+            check_type(argname="argument target_type", value=target_type, expected_type=type_hints["target_type"])
+        return typing.cast("Assignment", jsii.invoke(self, "grant", [principal, target_id, target_type]))
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, IPermissionSet).__jsii_proxy_class__ = lambda : _IPermissionSetProxy
@@ -419,17 +510,20 @@ class PermissionSet(
         self,
         principal: typing.Union["PrincipalProperty", typing.Dict[str, typing.Any]],
         target_id: builtins.str,
+        target_type: typing.Optional["TargetTypes"] = None,
     ) -> "Assignment":
         '''Grant this permission set to a given principal for a given targetId (AWS account identifier) on a given SSO instance.
 
         :param principal: -
         :param target_id: -
+        :param target_type: -
         '''
         if __debug__:
             type_hints = typing.get_type_hints(PermissionSet.grant)
             check_type(argname="argument principal", value=principal, expected_type=type_hints["principal"])
             check_type(argname="argument target_id", value=target_id, expected_type=type_hints["target_id"])
-        return typing.cast("Assignment", jsii.invoke(self, "grant", [principal, target_id]))
+            check_type(argname="argument target_type", value=target_type, expected_type=type_hints["target_type"])
+        return typing.cast("Assignment", jsii.invoke(self, "grant", [principal, target_id, target_type]))
 
     @builtins.property
     @jsii.member(jsii_name="cfnPermissionSet")
@@ -756,6 +850,11 @@ class PrincipalTypes(enum.Enum):
     GROUP = "GROUP"
 
 
+@jsii.enum(jsii_type="@renovosolutions/cdk-library-aws-sso.TargetTypes")
+class TargetTypes(enum.Enum):
+    AWS_ACCOUNT = "AWS_ACCOUNT"
+
+
 @jsii.implements(IAssignment)
 class Assignment(
     aws_cdk.Resource,
@@ -775,6 +874,7 @@ class Assignment(
         permission_set: IPermissionSet,
         principal: typing.Union[PrincipalProperty, typing.Dict[str, typing.Any]],
         target_id: builtins.str,
+        target_type: typing.Optional[TargetTypes] = None,
     ) -> None:
         '''
         :param scope: -
@@ -782,13 +882,17 @@ class Assignment(
         :param permission_set: The permission set to assign to the principal.
         :param principal: The principal to assign the permission set to.
         :param target_id: The target id the permission set will be assigned to.
+        :param target_type: The entity type for which the assignment will be created. Default: TargetTypes.AWS_ACCOUNT
         '''
         if __debug__:
             type_hints = typing.get_type_hints(Assignment.__init__)
             check_type(argname="argument scope", value=scope, expected_type=type_hints["scope"])
             check_type(argname="argument id", value=id, expected_type=type_hints["id"])
         props = AssignmentProps(
-            permission_set=permission_set, principal=principal, target_id=target_id
+            permission_set=permission_set,
+            principal=principal,
+            target_id=target_id,
+            target_type=target_type,
         )
 
         jsii.create(self.__class__, self, [scope, id, props])
@@ -807,6 +911,7 @@ __all__ = [
     "PermissionSetProps",
     "PrincipalProperty",
     "PrincipalTypes",
+    "TargetTypes",
 ]
 
 publication.publish()
